@@ -1,5 +1,3 @@
-
-
 se <- function(est,...) {
   UseMethod("se")
 }
@@ -7,8 +5,7 @@ se <- function(est,...) {
 se.LMbasic <- function(est, ...){
 
   check_der = FALSE
-  if(!is.null(est$sepiv))
-  {
+  if(!is.null(est$sePsi)){
     out = list(sepiv = est$sepiv,sePi = est$sePi,sePsi = est$sePsi)
     return(out)
   }
@@ -26,8 +23,7 @@ se.LMbasic <- function(est, ...){
 
   data.new <- data[,-c(id.which,tv.which), drop = FALSE]
 
-  if(is.null(responsesFormula))
-  {
+  if(is.null(responsesFormula)){
     Y <- data.new
     Xmanifest <- NULL
     Xinitial <- NULL
@@ -43,11 +39,12 @@ se.LMbasic <- function(est, ...){
     Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
   }
 
-  tmp <- long2matrices.internal(Y = Y, id = id, time = tv, yv = NULL,
+  yv = est$yv
+  tmp <- long2matrices.internal(Y = Y, id = id, time = tv, yv = yv,
                                 Xinitial = Xinitial, Xmanifest = Xmanifest, Xtrans = Xtrans)
   #model <- tmp$model
   S <- tmp$Y
-  yv = tmp$freq
+  # yv = tmp$freq
 
   miss = any(is.na(S))
   if(miss){
@@ -66,11 +63,20 @@ se.LMbasic <- function(est, ...){
   ns = sS[1]
   TT = sS[2]
 
+  if(k==1){
+    b = nrow(Psi)-1
+    r = ncol(Psi)
+    sePsi = matrix(0,b+1,r)
+    for(j in 1:r) sePsi[,j] = sqrt(diag(ginv(n*(diag(Psi[,j])-Psi[,j]%o%Psi[,j]))))
+    dimnames(sePsi)=list(category=0:b,item=1:r)
+    out = list(sePsi=sePsi)
+    return(out)
+  }
+
   if(length(sS)==2){
     r = 1
     if(is.matrix(S)) S = array(S,c(dim(S),1))
-  }else
-  {
+  }else{
     r = sS[3]
   }
 
@@ -97,9 +103,7 @@ se.LMbasic <- function(est, ...){
                     cbind(matrix(0,k-u,u-1),diag(k-u)))
   }
 
-
-
-  # Compute information matrix if required
+# Compute information matrix if required
   th = NULL
   for(u in 1:k) for(j in 1:r) th = c(th,m[[j]]$A%*%log(Psi[1:(bv[j]+1),u,j]))
   th = c(th,B%*%log(piv))
@@ -318,7 +322,9 @@ se.LMbasic <- function(est, ...){
   out = list(sepiv = sepiv, sePi = sePi,sePsi = sePsi)
   return(out)
 }
+
 se.LMlatent <- function(est,...){
+
   out_se = TRUE
   fort = TRUE
   check_der = FALSE
@@ -372,7 +378,7 @@ se.LMlatent <- function(est,...){
   X2 <- tmp$Xtrans
   S <- tmp$Y
   yv = tmp$freq
-
+  
   miss = any(is.na(S))
   if(miss){
     R = 1 * (!is.na(S))
@@ -385,6 +391,21 @@ se.LMlatent <- function(est,...){
   sS = dim(S)
   ns = sS[1]
   TT = sS[2]
+  
+  if(param=="difflogit"){
+    cat("\n* With difflogit is not possible to avoid the intercept for the transition probabilities*\n\n")
+    X2 = X2[,,-1,drop=FALSE]
+  }
+  
+  if(is.null(X1)){
+    X1 = matrix(1,ns,1)
+    colnames(X1) = "(Intercept)"
+  }
+  if(is.null(X2)){
+    X2 = array(1,c(ns,TT-1,1))
+    dimnames(X2) = list(NULL,NULL,"(Intercept)")
+  }
+  
   nc1 = dim(X1)[2]
   nc2 = dim(X2)[3]
 
@@ -441,9 +462,9 @@ se.LMlatent <- function(est,...){
     Xlab = out$label
   }
   Xndis = max(Xlab)
-  XXdis = array(0,c(k,(k-1)*(nc1+1),Xndis))
+  XXdis = array(0,c(k,(k-1)*nc1,Xndis))
   for(i in 1:Xndis){
-    if(nc1==0) xdis = 1 else xdis = c(1,Xdis[i,])
+    if(nc1==0) xdis = 1 else xdis = Xdis[i,]
     XXdis[,,i] = GBe%*%(diag(k-1)%x%t(xdis))
   }
 
@@ -459,8 +480,8 @@ se.LMlatent <- function(est,...){
     nameGa = NULL
     Zndis = max(Zlab)
   }else{
-    if(TT==2) X2 = array(X2,c(ns,1,dim(X2)[2]))
-    if(is.matrix(X2)) X2 = array(X2,c(ns,TT-1,1))
+   # if(TT==2) X2 = array(X2,c(ns,1,dim(X2)[2]))
+  #  if(is.matrix(X2)) X2 = array(X2,c(ns,TT-1,1))
     nc2 = dim(X2)[3] # number of covariates on the transition probabilities
     if(ns!= dim(X2)[1]) stop("dimension mismatch between S and X2")
 
@@ -472,7 +493,7 @@ se.LMlatent <- function(est,...){
     if(nc2==1) Zdis=matrix(Zdis,length(Zdis),1)
   }
   if(param=="multilogit"){
-    ZZdis = array(0,c(k,(k-1)*(nc2+1),Zndis,k))
+    ZZdis = array(0,c(k,(k-1)*nc2,Zndis,k))
     for(h in 1:k){
       if(k==2){
         if(h == 1) GGa = as.matrix(c(0,1)) else GGa = as.matrix(c(1,0))
@@ -480,7 +501,7 @@ se.LMlatent <- function(est,...){
         GGa = diag(k); GGa = GGa[,-h]
       }
       for(i in 1:Zndis){
-        if(nc2==0) zdis = 1 else zdis = c(1,Zdis[i,])
+        if(nc2==0) zdis = 1 else zdis = Zdis[i,]
         ZZdis[,,i,h] = GGa%*%(diag(k-1)%x%t(zdis))
       }
     }
@@ -509,6 +530,7 @@ se.LMlatent <- function(est,...){
   Be = est$Be
   Ga = est$Ga
   V = est$V
+  if(is.null(V)) stop("V must be available in the output of lmest (use output=TRUE option)")
 
   Am = vector("list",r)
   for(j in 1:r) Am[[j]] = rbind(rep(0,b[j]),diag(b[j]))
@@ -519,13 +541,14 @@ se.LMlatent <- function(est,...){
   # parameters on transition probabilities
   if(param=="multilogit"){
     if(is.list(Ga)) stop("invalid mode (list) for Ga")
-    Ga = matrix(Ga,(nc2+1)*(k-1),k)
+    Ga = matrix(Ga,nc2*(k-1),k)
     PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,ns,TT))
     for(h in 1:k){
       out =  prob_multilogit(ZZdis[,,,h],Ga[,h],Zlab,fort)
       PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
     }
   }else if(param=="difflogit"){
+    
     if(is.list(Ga)) Ga = c(as.vector(t(Ga[[1]])),as.vector(Ga[[2]]))
     if(length(Ga)!=k*(k-1)+(k-1)*nc2) stop("invalid dimensions for Ga")
     PI = array(0,c(k,k,ns,TT))
@@ -544,8 +567,9 @@ se.LMlatent <- function(est,...){
     count = count+b[j]
     th = c(th,log(temp[-1]/temp[1]))
   }
-  dlPiv = array(0,c(ns,k,(1+nc1)*(k-1)))
+  dlPiv = array(0,c(ns,k,nc1*(k-1)))
   be = as.vector(Be)
+
   out =  est_multilogit(V[,,1],XXdis,Xlab,be,Pivdis,fort=fort)
   Pivdis = out$Pdi
   for(j in 1:Xndis){
@@ -557,19 +581,19 @@ se.LMlatent <- function(est,...){
 
   count = 0
   if(param=="multilogit"){
-    dlPI = array(0,c(k,k,ns*TT,(1+nc2)*(k-1)*k))
+    dlPI = array(0,c(k,k,ns*TT,nc2*(k-1)*k))
     temp0 = rep(1,k); Temp0 = diag(k)
     for(h in 1:k){
-      ind = count+(1:((1+nc2)*(k-1)))
+      ind = count+(1:(nc2*(k-1)))
       for(j in 1:Zndis){
         temp = pmax(PIdis[j,,h],10^-50)
         Temp = (Temp0-temp0%o%temp)%*%ZZdis[,,j,h]
         for(i in which(Zlab==j)) dlPI[h,,ns+i,ind] = Temp
       }
-      count = count+((1+nc2)*(k-1))
+      count = count+(nc2*(k-1))
       th = c(th,Ga[,h])
     }
-    dlPI = array(dlPI,c(k,k,ns,TT,(1+nc2)*(k-1)*k))
+    dlPI = array(dlPI,c(k,k,ns,TT,nc2*(k-1)*k))
   }else if(param=="difflogit"){
     dlPI = array(0,c(k,k*ns*TT,(k+nc2)*(k-1)))
     temp0 = rep(1,k); Temp0 = diag(k)
@@ -633,7 +657,7 @@ se.LMlatent <- function(est,...){
     for(h in 1:k){
       UU = NULL
       for(t in 2:TT) UU = rbind(UU,t(U[h,,,t]))
-      tmp = ZZdis[,,,h]
+      tmp = array(ZZdis[,,,h],dim(ZZdis)[1:3]) #SP: tmp = ZZdis[,,,h]
       if(nc2==0) tmp = array(tmp,c(k,(k-1),Zndis))
       tmp2 = PIdis[,,h]
       if(Zndis==1) tmp2 = matrix(tmp2,1,k)
@@ -685,7 +709,7 @@ se.LMlatent <- function(est,...){
       for(h1 in 1:npar){
         UU = NULL
         for(t in 2:TT) UU = rbind(UU,t(dU[h,,,t,h1]))
-        tmp = ZZdis[,,,h]
+        tmp = array(ZZdis[,,,h],dim(ZZdis)[1:3]) #SP: tmp = ZZdis[,,,h]
         if(nc2==0) tmp = array(tmp,c(k,(k-1),Zndis))
         tmp2 = PIdis[,,h]
         if(Zndis==1) tmp2 = matrix(tmp2,1,k)
@@ -753,15 +777,13 @@ se.LMlatent <- function(est,...){
     }
   }
 
-  if(out_se) {seBe = matrix(sebe,nc1+1,k-1); dimnames(seBe) = list(nameBe,logit=2:k)}
+  if(out_se) {seBe = matrix(sebe,nc1,k-1); dimnames(seBe) = list(nameBe,logit=2:k)}
   if(param=="multilogit"){
     if(is.null(nameGa)){
-      if(nc2==0) nameGa = c("Intercept") else nameGa = c("intercept", paste("X2",1:nc2,sep=""))
-    }else{
-      nameGa = c("intercept",nameGa)
+      if(nc2==0) nameGa = "(Intercept)" else nameGa = c("(Intercept)", paste("X2",1:(nc2-1),sep=""))
     }
     if(k>2) {
-      Ga = array(as.vector(Ga),c(nc2+1,k-1,k))
+      Ga = array(as.vector(Ga),c(nc2,k-1,k))
       dimnames(Ga) = list(nameGa,logit=2:k,logit=1:k)
     }else if(k==2){
 
@@ -769,10 +791,10 @@ se.LMlatent <- function(est,...){
     }
     if(out_se){
       if(k==2){
-        seGa = matrix(sega,nc2+1,2)
+        seGa = matrix(sega,nc2,2)
         dimnames(seGa) = list(nameGa,logit=1:k)
       }else if(k>2){
-        seGa = array(as.vector(sega),c(nc2+1,k-1,k))
+        seGa = array(as.vector(sega),c(nc2,k-1,k))
         dimnames(seGa) = list(nameGa,logit=2:k,logit=1:k)
       }
     }
@@ -788,20 +810,20 @@ se.LMlatent <- function(est,...){
       nameGa2 = nameGa
     }
     if (k==2) {
-      dimnames(Ga[[1]]) = list(intercept=1:k,logit=k)
+      dimnames(Ga[[1]]) = list("(Intercept)"=1:k,logit=k)
       dimnames(Ga[[2]])=list(nameGa2,logit=k)
     } else if (k>2){
-      dimnames(Ga[[1]]) = list(intercept=1:k,logit=2:k)
+      dimnames(Ga[[1]]) = list("(Intercept)"=1:k,logit=2:k)
       dimnames(Ga[[2]])=list(nameGa2,logit=2:k)
     }
     if(out_se){
       seGa[[1]] = t(matrix(sega[1:(k*(k-1))],k-1,k))
       seGa[[2]] = matrix(sega[(k*(k-1))+(1:((k-1)*nc2))],nc2,k-1)
       if(k==2){
-        dimnames(seGa[[1]]) = list(intercept=1:k,logit=k)
+        dimnames(seGa[[1]]) = list("(Intercept)"=1:k,logit=k)
         dimnames(seGa[[2]])=list(nameGa2,logit=k)
       }else if (k>2){
-        dimnames(seGa[[1]]) = list(intercept=1:k,logit=2:k)
+        dimnames(seGa[[1]]) = list("(Intercept)"=1:k,logit=2:k)
         dimnames(seGa[[2]])=list(nameGa2,logit=2:k)
       }
     }
@@ -810,12 +832,10 @@ se.LMlatent <- function(est,...){
   out = list(sePsi = sePsi, seBe = seBe,seGa = seGa)
   return(out)
 }
+
 se.LMbasiccont <- function(est,...){
 
-
-
-  if(!is.null(est$seMu))
-  {
+  if(!is.null(est$seMu)){
     out = list(sepiv = est$sepiv,sePi = est$sePi,seMu = est$seMu,seSi = est$seSi)
     return(out)
   }
@@ -833,8 +853,7 @@ se.LMbasiccont <- function(est,...){
 
   data.new <- data[,-c(id.which,tv.which), drop = FALSE]
 
-  if(is.null(responsesFormula))
-  {
+  if(is.null(responsesFormula)){
     Y <- data.new
     Xmanifest <- NULL
     Xinitial <- NULL
@@ -846,24 +865,27 @@ se.LMbasiccont <- function(est,...){
     Xinitial <- NULL
     Xtrans <- NULL
   }
-  if(min(Y,na.rm=T)>0){
-    Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
-  }
+  if(min(Y,na.rm=T)>0) Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
 
-  tmp <-  long2matrices.internal(Y = Y, id = id, time = tv, yv = NULL,
+  tmp <-  long2matrices.internal(Y = Y, id = id, time = tv, yv = est$yv,
                                         Xinitial = Xinitial, Xmanifest = Xmanifest, Xtrans = Xtrans)
   #model <- tmp$model
   Y <- tmp$Y
-
+  yv = est$yv
+  ns = dim(Y)[1]
+  TT = dim(Y)[2]
+  r = dim(Y)[3]
+  n = sum(yv)
   miss = any(is.na(Y))
 
   if(miss){
+    Yv = matrix(Y,ns*TT,r)
     Yv = cbind(1,Yv)
     pYv = prelim.mix(Yv,1)
     thhat = em.mix(prelim.mix(Yv,1))
     rngseed(1)
     Yv = as.matrix(imp.mix(pYv, da.mix(pYv,thhat,steps=100), Yv)[,-1])
-    Y = array(Yv,c(n,TT,r))
+    Y = array(Yv,c(ns,TT,r))
     cat("Missing data in the dataset. imp.mix function (mix package) used for imputation.\n")
   }
 
@@ -872,21 +894,22 @@ se.LMbasiccont <- function(est,...){
   Mu = est$Mu
   Si = est$Si
 
-
-
   sY = dim(Y)
-  n = sY[1]
-  TT = sY[2]
+  ns = as.integer(sY[1])
+  TT = as.integer(sY[2])
+  n = sum(yv)
 
   if(length(sY)==2){
     r = 1
     if(is.matrix(Y)) Y = array(Y,c(dim(Y),1))
-  }else r = sY[3]
+  }else{
+    r = sY[3]
+  }
 
-  Yv = matrix(Y,n*TT,r)
+  Yv = matrix(Y,ns*TT,r)
 
   th = NULL; sc = NULL; J = NULL
-
+  
   B = cbind(-rep(1,k-1),diag(k-1))
   Bm = rbind(rep(0,k-1),diag(k-1))
   C = array(0,c(k-1,k,k))
@@ -898,25 +921,24 @@ se.LMbasiccont <- function(est,...){
                     rep(0,k-1),
                     cbind(matrix(0,k-u,u-1),diag(k-u)))
   }
-
-
-
   th = NULL
   th = c(th,as.vector(Mu))
   th = c(th,Si[upper.tri(Si,TRUE)])
-  th = c(th,B%*%log(piv))
-  if(mod==0) for(t in 2:TT) for(u in 1:k) th = c(th,C[,,u]%*%log(Pi[u,,t]))
-  if(mod==1) for(u in 1:k) th = c(th,C[,,u]%*%log(Pi[u,,2]))
+  if(k>1){
+    th = c(th,B%*%log(piv))
+    if(mod==0) for(t in 2:TT) for(u in 1:k) th = c(th,C[,,u]%*%log(Pi[u,,t]))
+    if(mod==1) for(u in 1:k) th = c(th,C[,,u]%*%log(Pi[u,,2]))
+  }
 
   th0 = th-10^-5/2
-  out =  lk_obs_cont(th0,Bm,Cm,k,Y,TT,r,mod)
+  out =  lk_obs_cont(th0,yv,Bm,Cm,k,Y,TT,r,mod)
   lk0 = out$lk; sc0 = out$sc
   lth = length(th)
   scn = rep(0,lth)
   J = matrix(0,lth,lth)
   for(j in 1:lth){
     thj = th0; thj[j] = thj[j]+10^-5
-    out =  lk_obs_cont(thj,Bm,Cm,k,Y,TT,r,mod)
+    out =  lk_obs_cont(thj,yv,Bm,Cm,k,Y,TT,r,mod)
     scn[j] = (out$lk-lk0)/10^-5
     J[,j] = (out$sc-sc0)/10^-5
   }
@@ -927,84 +949,88 @@ se.LMbasiccont <- function(est,...){
   Va2 = Va[1:(nMu+nSi),1:(nMu+nSi)]
   se2 = sqrt(diag(Va2))
 
-  Va = Va[-(1:(nMu+nSi)),-(1:(nMu+nSi))]
-  Om = diag(piv)-tcrossprod(piv,piv)
-  M = Om%*%Bm
-  if(mod==0){
-    for(t in 2:TT) for(u in 1:k){
-      Om = diag(Pi[u,,t])-Pi[u,,t]%o%Pi[u,,t]
-      M =  blkdiag(M,Om%*%Cm[,,u])
+  if(k>1){
+    Va = Va[-(1:(nMu+nSi)),-(1:(nMu+nSi))]
+    Om = diag(piv)-tcrossprod(piv,piv)
+    M = Om%*%Bm
+    if(mod==0){
+      for(t in 2:TT) for(u in 1:k){
+        Om = diag(Pi[u,,t])-Pi[u,,t]%o%Pi[u,,t]
+        M =  blkdiag(M,Om%*%Cm[,,u])
+      }
     }
+    if(mod==1){
+      for(u in 1:k){
+        Om = diag(Pi[u,,2])-Pi[u,,2]%o%Pi[u,,2]
+        M =  blkdiag(M,Om%*%Cm[,,u])
+      }
+    }
+    if(mod>1){
+      for(u in 1:k){
+        Om = diag(Pi[u,,2])-Pi[u,,2]%o%Pi[u,,2]
+        M =  blkdiag(M,Om%*%Cm[,,u])
+      }
+      for(u in 1:k){
+        Om = diag(Pi[u,,mod+1])-Pi[u,,mod+1]%o%Pi[u,,mod+1]
+        M =  blkdiag(M,Om%*%Cm[,,u])
+      }
+    }
+    M = as.matrix(M)
+    Va = M%*%Va%*%t(M)
+    dVa = diag(Va)
+    if(any(dVa<0)) warning("Negative elements in the estimated variance-covariance matrix for the parameters estimates")
+    se = sqrt(abs(dVa))
   }
-  if(mod==1){
-    for(u in 1:k){
-      Om = diag(Pi[u,,2])-Pi[u,,2]%o%Pi[u,,2]
-      M =  blkdiag(M,Om%*%Cm[,,u])
-    }
-  }
-  if(mod>1){
-    for(u in 1:k){
-      Om = diag(Pi[u,,2])-Pi[u,,2]%o%Pi[u,,2]
-      M =  blkdiag(M,Om%*%Cm[,,u])
-    }
-    for(u in 1:k){
-      Om = diag(Pi[u,,mod+1])-Pi[u,,mod+1]%o%Pi[u,,mod+1]
-      M =  blkdiag(M,Om%*%Cm[,,u])
-    }
-  }
-  M = as.matrix(M)
-  Va = M%*%Va%*%t(M)
-  dVa = diag(Va)
-  if(any(dVa<0)) warning("Negative elements in the estimated variance-covariance matrix for the parameters estimates")
-  se = sqrt(abs(dVa))
-  # Divide parameters
-  se = c(se2,se)
+
+# Divide parameters
+  if(k==1) se = se2 else se = c(se2,se)
   seMu = se[1:nMu]
   seSi = se[nMu+(1:nSi)]
-  sepiv = se[nMu+nSi+(1:k)]
-
-  if(mod==0) sePi = se[nMu+nSi+k+(1:(k*k*(TT-1)))]
-  if(mod==1) sePi = se[nMu+nSi+k+(1:(k*k))]
-  if(mod>1) sePi = se[nMu+nSi+k+(1:(k*k*2))]
-
-
+  if(k==1){
+    sepiv = NULL; sePi = NULL
+  }else{
+    sepiv = se[nMu+nSi+(1:k)]
+    if(mod==0) sePi = se[nMu+nSi+k+(1:(k*k*(TT-1)))]
+    if(mod==1) sePi = se[nMu+nSi+k+(1:(k*k))]
+    if(mod>1) sePi = se[nMu+nSi+k+(1:(k*k*2))]
+  }
 
   if(miss) out$Y = Y
 
-
   seMu = matrix(seMu,r,k)
   seSi2 = matrix(0,r,r)
   seSi2[upper.tri(seSi2,TRUE)]=seSi
-  seSi2 = seSi2+t(seSi2-diag(diag(seSi2)))
+  if(nrow(seSi2)>1) seSi2 = seSi2+t(seSi2-diag(diag(seSi2)))
   seSi = seSi2
-  sePi0 = sePi
-  sePi = array(0,c(k,k,TT))
-  if(mod>1){
-    sePi0 = array(sePi0,c(k,k,2))
-    sePi0 = aperm(sePi0,c(2,1,3))
-    sePi[,,2:mod] = sePi0[,,1]
-    sePi[,,(mod+1):TT] = sePi0[,,2]
-  } else {
-    sePi[,,2:TT] = sePi0
-    sePi = aperm(sePi,c(2,1,3))
+  if(k>1){
+    sePi0 = sePi
+    sePi = array(0,c(k,k,TT))
+    if(mod>1){
+      sePi0 = array(sePi0,c(k,k,2))
+      sePi0 = aperm(sePi0,c(2,1,3))
+      sePi[,,2:mod] = sePi0[,,1]
+      sePi[,,(mod+1):TT] = sePi0[,,2]
+    } else {
+      sePi[,,2:TT] = sePi0
+      sePi = aperm(sePi,c(2,1,3))
+    }
+    dimnames(sePi) = list(state=1:k,state=1:k,time=1:TT)
   }
-  dimnames(sePi) = list(state=1:k,state=1:k,time=1:TT)
   if(r==1) dimnames(seMu) = list(item=1,state=1:k) else dimnames(seMu)=list(item=1:r,state=1:k)
 
   out = list(sepiv = sepiv,sePi = sePi,seMu = seMu,seSi = seSi)
-
   return(out)
 
-
 }
+
 se.LMlatentcont <- function(est,...){
+
   out_se = TRUE
   #fort = TRUE
   #check_der = FALSE
 
-  if(!is.null(est$seBe))
-  {
-    out = list(seMu = out$seMu,seSi = out$seSi, seBe = est$seBe,seGa = est$seGa)
+  if(!is.null(est$seBe)){
+    out = list(seMu = est$seMu,seSi = est$seSi, seBe = est$seBe,seGa = est$seGa)
     return(out)
   }
 
@@ -1021,8 +1047,7 @@ se.LMlatentcont <- function(est,...){
 
   data.new <- data[,-c(id.which,tv.which), drop = FALSE]
 
-  if(is.null(responsesFormula))
-  {
+  if(is.null(responsesFormula)){
     Y <- data.new
     Xmanifest <- NULL
     Xinitial <- NULL
@@ -1034,12 +1059,9 @@ se.LMlatentcont <- function(est,...){
     Xinitial <- NULL
     Xtrans <- NULL
   }
-  if(min(Y,na.rm=T)>0){
-    Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
-  }
+  if(min(Y,na.rm=T)>0) Y <- apply(Y,2,function(x) x - min(x, na.rm = TRUE))
 
-  if(!is.null(latentFormula))
-  {
+  if(!is.null(latentFormula)){
     temp <-  getLatent(data = data.new,latent = latentFormula, responses = responsesFormula)
     Xinitial <- temp$Xinitial
     Xtrans <- temp$Xtrans
@@ -1050,18 +1072,22 @@ se.LMlatentcont <- function(est,...){
   X1 <- tmp$Xinitial
   X2 <- tmp$Xtrans
   Y <- tmp$Y
-  yv = tmp$freq
-
+  R = array(1,dim(Y))
+  yv = est$yv
 
   miss = any(is.na(Y))
 
   if(miss){
+    ns = dim(Y)[1]
+    TT = dim(Y)[2]
+    r = dim(Y)[3]
+    Yv = matrix(Y,ns*TT,r)
     Yv = cbind(1,Yv)
     pYv = prelim.mix(Yv,1)
     thhat = em.mix(prelim.mix(Yv,1))
     rngseed(1)
     Yv = as.matrix(imp.mix(pYv, da.mix(pYv,thhat,steps=100), Yv)[,-1])
-    Y = array(Yv,c(n,TT,r))
+    Y = array(Yv,c(ns,TT,r))
     cat("Missing data in the dataset. imp.mix function (mix package) used for imputation.\n")
   }
 
@@ -1070,18 +1096,18 @@ se.LMlatentcont <- function(est,...){
   Mu = est$Mu
   Si = est$Si
 
-
-
   sY = dim(Y)
-  n = sY[1]
-  TT = sY[2]
+  ns = as.integer(sY[1])
+  TT = as.integer(sY[2])
+  n = sum(yv)
 
   if(length(sY)==2){
     r = 1
     if(is.matrix(Y)) Y = array(Y,c(dim(Y),1))
-  }else r = sY[3]
-
-  Yv = matrix(Y,n*TT,r)
+  }else{
+    r = sY[3]
+  }
+  Yv = matrix(Y,ns*TT,r)
 
   if(k == 2){
     GBe = as.matrix(c(0,1))
@@ -1089,9 +1115,9 @@ se.LMlatentcont <- function(est,...){
     GBe = diag(k); GBe = GBe[,-1]
   }
 
-  if(is.vector(X1)) X1 = matrix(X1,n,1)
+  if(is.vector(X1)) X1 = matrix(X1,ns,1)
   nc1 = dim(X1)[2] # number of covariates on the initial probabilities
-  if(n!= dim(X1)[1]) stop("dimension mismatch between S and X1")
+  if(ns!= dim(X1)[1]) stop("dimension mismatch between S and X1")
   nameBe = colnames(X1)
   out =  aggr_data(X1)
   Xdis = out$data_dis
@@ -1099,15 +1125,15 @@ se.LMlatentcont <- function(est,...){
   Xlab = out$label
 
   Xndis = max(Xlab)
-  XXdis = array(0,c(k,(k-1)*(nc1+1),Xndis))
+  XXdis = array(0,c(k,(k-1)*nc1,Xndis))
   for(i in 1:Xndis){
-    if(nc1==0) xdis = 1 else xdis = c(1,Xdis[i,])
+    if(nc1==0) xdis = 1 else xdis = Xdis[i,]
     XXdis[,,i] = GBe%*%(diag(k-1)%x%t(xdis))
   }
-  if(TT==2) X2 = array(X2,c(n,1,dim(X2)[2]))
-  if(is.matrix(X2)) X2 = array(X2,c(n,TT-1,1))
+  if(TT==2) X2 = array(X2,c(ns,1,dim(X2)[2]))
+  if(is.matrix(X2)) X2 = array(X2,c(ns,TT-1,1))
   nc2 = dim(X2)[3] # number of covariates on the transition probabilities
-  if(n!= dim(X2)[1]) stop("dimension mismatch between S and X2")
+  if(ns!= dim(X2)[1]) stop("dimension mismatch between S and X2")
   nameGa = colnames(aperm(X2,c(1,3,2)))
   Z = NULL
   for(t in 1:(TT-1)) Z = rbind(Z,X2[,t,])
@@ -1116,7 +1142,7 @@ se.LMlatentcont <- function(est,...){
   if(nc2==1) Zdis=matrix(Zdis,length(Zdis),1)
 
   if(param=="multilogit"){
-    ZZdis = array(0,c(k,(k-1)*(nc2+1),Zndis,k))
+    ZZdis = array(0,c(k,(k-1)*nc2,Zndis,k))
     for(h in 1:k){
       if(k==2){
         if(h == 1) GGa = as.matrix(c(0,1)) else GGa = as.matrix(c(1,0))
@@ -1124,12 +1150,12 @@ se.LMlatentcont <- function(est,...){
         GGa = diag(k); GGa = GGa[,-h]
       }
       for(i in 1:Zndis){
-        if(nc2==0) zdis = 1 else zdis = c(1,Zdis[i,])
+        if(nc2==0) zdis = 1 else zdis = Zdis[i,]
         ZZdis[,,i,h] = GGa%*%(diag(k-1)%x%t(zdis))
       }
     }
   }else if(param=="difflogit"){
-    Zlab = (((Zlab-1)*k)%x%rep(1,k))+rep(1,n*(TT-1))%x%(1:k)
+    Zlab = (((Zlab-1)*k)%x%rep(1,k))+rep(1,ns*(TT-1))%x%(1:k)
     ZZdis = array(0,c(k,k*(k-1)+(k-1)*nc2,Zndis*k))
     j = 0
     for(i in 1:Zndis){
@@ -1154,23 +1180,21 @@ se.LMlatentcont <- function(est,...){
   # parameters on transition probabilities
   if(param=="multilogit"){
     if(is.list(Ga)) stop("invalid mode (list) for Ga")
-    Ga = matrix(Ga,(nc2+1)*(k-1),k)
-    PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,n,TT))
+    Ga = matrix(Ga,nc2*(k-1),k)
+    PIdis = array(0,c(Zndis,k,k)); PI = array(0,c(k,k,ns,TT))
     for(h in 1:k){
       out =  prob_multilogit(ZZdis[,,,h],Ga[,h],Zlab)
-      PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,n,TT-1))
+      PIdis[,,h] = out$Pdis; PI[h,,,2:TT] = array(as.vector(t(out$P)),c(1,k,ns,TT-1))
     }
   }else if(param=="difflogit"){
     if(is.list(Ga)) Ga = c(as.vector(t(Ga[[1]])),as.vector(Ga[[2]]))
     if(length(Ga)!=k*(k-1)+(k-1)*nc2) stop("invalid dimensions for Ga")
     PI = array(0,c(k,k,n,TT))
     out =  prob_multilogit(ZZdis,Ga,Zlab)
-    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(k,k,n,TT-1))
+    PIdis = out$Pdis; PI[,,,2:TT] = array(as.vector(t(out$P)),c(k,k,ns,TT-1))
     PI = aperm(PI,c(2,1,3,4))
   }
 
-
-
   th = NULL
   th = c(th,as.vector(Mu))
   th = c(th,Si[upper.tri(Si,TRUE)])
@@ -1178,8 +1202,7 @@ se.LMlatentcont <- function(est,...){
   if(param=="multilogit"){
     for(h in 1:k) th = c(th, Ga[,h])
   }else if(param=="difflogit") th = c(th,Ga)
-
-  out =  lk_obs_latent_cont(th,Y,yv,XXdis,Xlab,ZZdis,Zlab,param)
+  out = lk_obs_latent_cont(th,Y,R,yv,XXdis,Xlab,ZZdis,Zlab,param)
   lk0 = out$lk; sc0 = out$sc
 
   lth = length(th)
@@ -1187,7 +1210,7 @@ se.LMlatentcont <- function(est,...){
 
   for(h in 1:lth){
     thh = th; thh[h] = thh[h]+10^-5
-    outh =  lk_obs_latent_cont(thh,Y,yv,XXdis,Xlab,ZZdis,Zlab,param)
+    outh =  lk_obs_latent_cont(thh,Y,R,yv,XXdis,Xlab,ZZdis,Zlab,param)
     scn[h] = (outh$lk-lk0)/10^-5
     Fn[,h] = (outh$sc-sc0)/10^-5
   }
@@ -1198,29 +1221,25 @@ se.LMlatentcont <- function(est,...){
 
   nMu = r*k
   nSi = r*(r+1)/2
-  nbe = (1+nc1)*(k-1)
-  if(param=="multilogit") nga=(1+nc2)*(k-1)*k else if(param=="difflogit") nga=(k+nc2)*(k-1)
+  nbe = nc1*(k-1)
+  if(param=="multilogit") nga=nc2*(k-1)*k else if(param=="difflogit") nga=(k+nc2)*(k-1)
   seMu = se[1:nMu]
   seSi = se[nMu+(1:nSi)]
   sebe = se[nMu+nSi+(1:nbe)]
   sega = se[nMu+nSi+nbe+(1:nga)]
 
   if (is.null(nameBe)){
-    if(nc1==0) nameBe = c("Intercept") else nameBe = c("intercept",paste("X1",1:nc1,sep=""))
-  }else{
-    nameBe = c("intercept",nameBe)
+    if(nc1==0) nameBe = "(Intercept)" else nameBe = c("(Intercept)",paste("X1",1:(nc1-1),sep=""))
   }
-  seBe = matrix(sebe,nc1+1,k-1); dimnames(seBe) = list(nameBe,logit=2:k)
+  seBe = matrix(sebe,nc1,k-1); dimnames(seBe) = list(nameBe,logit=2:k)
 
 
   if(param=="multilogit"){
     if(is.null(nameGa)){
-      if(nc2==0) nameGa = c("Intercept") else nameGa = c("intercept", paste("X2",1:nc2,sep=""))
-    }else{
-      nameGa = c("intercept",nameGa)
+      if(nc2==0) nameGa = "(Intercept)" else nameGa = c("(Intercept)", paste("X2",1:(nc2-1),sep=""))
     }
     if(k>2) {
-      Ga = array(as.vector(Ga),c(nc2+1,k-1,k))
+      Ga = array(as.vector(Ga),c(nc2,k-1,k))
       dimnames(Ga) = list(nameGa,logit=2:k,logit=1:k)
     }else if(k==2){
 
@@ -1228,10 +1247,10 @@ se.LMlatentcont <- function(est,...){
     }
     if(out_se){
       if(k==2){
-        seGa = matrix(sega,nc2+1,2)
+        seGa = matrix(sega,nc2,2)
         dimnames(seGa) = list(nameGa,logit=1:k)
       }else if(k>2){
-        seGa = array(as.vector(sega),c(nc2+1,k-1,k))
+        seGa = array(as.vector(sega),c(nc2,k-1,k))
         dimnames(seGa) = list(nameGa,logit=2:k,logit=1:k)
       }
     }
@@ -1247,20 +1266,20 @@ se.LMlatentcont <- function(est,...){
       nameGa2 = nameGa
     }
     if (k==2) {
-      dimnames(Ga[[1]]) = list(intercept=1:k,logit=k)
+      dimnames(Ga[[1]]) = list("(Intercept)"=1:k,logit=k)
       dimnames(Ga[[2]])=list(nameGa2,logit=k)
     } else if (k>2){
-      dimnames(Ga[[1]]) = list(intercept=1:k,logit=2:k)
+      dimnames(Ga[[1]]) = list("(Intercept)"=1:k,logit=2:k)
       dimnames(Ga[[2]])=list(nameGa2,logit=2:k)
     }
     if(out_se){
       seGa[[1]] = t(matrix(sega[1:(k*(k-1))],k-1,k))
       seGa[[2]] = matrix(sega[(k*(k-1))+(1:((k-1)*nc2))],nc2,k-1)
       if(k==2){
-        dimnames(seGa[[1]]) = list(intercept=1:k,logit=k)
+        dimnames(seGa[[1]]) = list("(Intercept)"=1:k,logit=k)
         dimnames(seGa[[2]])=list(nameGa2,logit=k)
       }else if (k>2){
-        dimnames(seGa[[1]]) = list(intercept=1:k,logit=2:k)
+        dimnames(seGa[[1]]) = list("(Intercept)"=1:k,logit=2:k)
         dimnames(seGa[[2]])=list(nameGa2,logit=2:k)
       }
     }
@@ -1270,7 +1289,7 @@ se.LMlatentcont <- function(est,...){
   if(r==1) dimnames(seMu) = list(item=1,state=1:k) else dimnames(seMu)=list(item=1:r,state=1:k)
   seSi2 = matrix(0,r,r)
   seSi2[upper.tri(seSi2,TRUE)]=seSi
-  seSi2 = seSi2+t(seSi2-diag(diag(seSi2)))
+  if(nrow(seSi2)>1) seSi2 = seSi2+t(seSi2-diag(diag(seSi2)))
   seSi = seSi2
   dimnames(seSi)=list(item=1:r,item=1:r)
 
